//***************************************************************************
//  File........: bcd.c
//  Target(s)...: AVR 
//  Compiler....: AVR-GCC
//  Description.: BCD conversion algorithms
//***************************************************************************

const unsigned int unit[]={10000,1000,100,10};
#define UNIT_TEN_THOU	0
#define UNIT_THOU		1
#define UNIT_HUNDRED	2
#define UNIT_TEN		3


/*****************************************************************************
*   Function name : convertBCD
*   Returns :       Binary coded decimal value of the input 
*   Parameters :    input: Value between (0-65535) to be encoded into BCD 
*					digit: Highest digit of the conversion (from defines above)
*   Purpose :       Convert a number into a BCD encoded character.
*                   wandelt Zahl in BCD-codierte Zeichen um
*                   kann nur bis 65535 arbeiten, da 
*                   Integer unsigned =16bit bis 65535
*****************************************************************************/
unsigned long convertBCD(char digit, unsigned int input)
{
	// z.B. 65535 wandeln -> convertBCD(UNIT_TEN_THOU,65535)
	// z.B. 9999 wandeln -> convertBCD (UNIT_THOU,9999)
	// z.B. 999 wandeln -> convertBCD(UNIT_HUNDRED,999)   
	// z.B. 99 wandeln -> convertBCD(UNIT_TEN,99) 

	unsigned long high = 0;  // long erlaubt grosse Zahlen 0 bis mehr als 65536
	char i;
	
	for (i=digit;i<=UNIT_TEN;i++){       // i=digit bis 3, da UNIT_TEN=3
		while (input >= unit[(int)i]){   // unit (0)=10000, 
										 // unit (1)=1000, 
										 // unit (2)=100, 
										 // unit (3)=10  
			high++;
			input -= unit[(int)i];
		}
		
		high <<= 4;  // eine Stelle links= x2, 2 Stellen= x4, 3 Stellen =x8, 4 Stellen =x16
		             // nchstes Halbbyte bearbeiten 
	}
	
    return  high | input;        // Add ones and return answer
    
    // Beispiel 99 wandeln -> convertBCD(3,99) 
    // digit=3 -> nur ein Mal in der for-Schleife
    // input=99 unit(3)=10 -> high=1, input=89
    //                     ->      2        79
    //                     ->      3        69
    //                     ->      4        59
    //                     ->      5        49
    //                     ->      6        39
    //                     ->      7        29
    //                     ->      8        19
    //                     ->      9        09  Abbruch
    // high=0x90
    // return 0x90 | 0x09 verodert -> 0x99 kann nun hex ausgegeben werden
    //
    // Beispiel 999 wandeln -> convertBCD(2,999) 
    // digit=2 -> genau 2x in der for-Schleife
    // input=999 unit(2)=100 -> high=1, input=899
    //                       ->      2        799
    //                       ->      3        699
    //                       ->      4        599
    //                       ->      5        499
    //                       ->      6        399
    //                       ->      7        299
    //                       ->      8        199
    //                       ->      9         99 Abbruch
    // high=0x90
    // input=99 unit(3)=10 -> high=0x91, input=89
    //                     ->      0x92        79
    //                     ->      0x93        69
    //                     ->      0x94        59
    //                     ->      0x95        49
    //                     ->      0x96        39
    //                     ->      0x97        29
    //                     ->      0x98        19
    //                     ->      0x99        09 Abbruch
    // high=0x990
    // return 0x990 | 0x09 verodert -> 0x999 kann nun hex ausgegeben werden
    //
    // Beispiel 1023 wandeln -> convertBCD(2,1023) 
    // digit=2 -> genau 2x in der for-Schleife
    // input=1023 unit(2)=100 -> high=1, input=923
    //                        ->      2        823
    //                        ->      3        723
    //                        ->      4        623
    //                        ->      5        523
    //                        ->      6        423
    //                        ->      7        323
    //                        ->      8        223
    //                        ->      9        123
    //                        ->     10        023 Abbruch
    // high=0xA0
    // input=23 unit(3)=10 -> high=0xA1, input=13
    //                     ->      0xA2        03                   
    // high=0xA20
    // return 0xA20 | 0x03 verodert -> 0xA23 kann nun hex ausgegeben werden
    //
    // Beispiel 65535 (0xFFFF) wandeln -> convertBCD(0,65535) 
    // digit=0 -> genau 4x in der for-Schleife
    // input=65535 unit(0)=10000 -> high=1, input=55535
    //                           ->      2        45535
    //                           ->      3        35535
    //                           ->      4        25535
    //                           ->      5        15535
    //                           ->      6        05535
    // high=0x60
    // input=5535 unit(1)=1000 -> high=0x61, input=4535
    //                         ->      0x62        3535
    //                         ->      0x63        2535
    //                         ->      0x64        1535
    //                         ->      0x65        0535                
    // high=0x650
    // input=535 unit(2)=100 -> high=0x651, input=435
    //                       ->      0x652        335
    //                       ->      0x653        235
    //                       ->      0x654        135
    //                       ->      0x655        035
    // high=0x6550
    // input=35 unit(3)=10 -> high=0x6551, input=25
    //                     ->      0x6552        15
    //                     ->      0x6553        05
    // return 0x65530 | 0x05 verodert -> 0x65535 kann nun hex ausgegeben werden
    //                                     0110 0101 0101 0011 0101 = 20bit >16bit !
    //
    // Beispiel 9999 wandeln -> convertBCD(1,9999) 
    // 3x in der Schleife
    // return 0x9990 | 0x09 verodert -> 0x9999 kann nun hex ausgegeben werden
    //                                    1010 1010 1010 1010 = 16bit
}


/*****************************************************************************
*   Function name : CHAR2BCD2
*   Returns :       Binary coded decimal value of the input (2 digits)
*   Parameters :    Value between (0-99) to be encoded into BCD 
*   Purpose :       Convert a character into a BCD encoded character.
*                   The input must be in the range 0 to 99.
*                   The result is byte where the high and low nibbles
*                   contain the tens and ones of the input.

*                   ein Byte als 2 Dezimalziffern senden (nur 0-99 sinnvoll)
*                     0 (=0x00) als 0 0 senden
*                     1 (=0x01) als 0 1 senden
*                     9 (=0x09) als 0 9 senden
*                    10 (=0x0A) als 1 0 senden
*                    15 (=0x0F) als 1 5 senden
*                    16 (=0x10) als 1 6 senden
*                    99 (=0x63) als 9 9 senden
*                   100 (=0x64) als A 0 senden passt sonst ja nicht mehr in 2 Stellen
*                   101 (=0x65) als A 1 senden?
*                   109 (=0x6D) als A 9 senden?
*                   110 (=0x6E) als B 0 senden?
*                   120         als C 0 senden?
*                   130         als D 0 senden?
*                   140         als E 0 senden?
*                   150         als F 0 senden?
*                   151         als F 1 senden?
*                   159         als F 9 senden?
*                   255 (=0xFF) als ???
*****************************************************************************/
char CHAR2BCD2(char input)
{
	// unsigned char 0-99 umwandeln in unsigned char 2 BCD-Ziffern 00 bis 99
	// UNIT_TEN=3 z.B. 99 wandeln -> convertBCD(3,99) 
	
	return (char) convertBCD(UNIT_TEN,(unsigned int) input);
}


/*****************************************************************************
*   Function name : INT2BCD3
*   Returns :       Binary coded decimal value of the input (3 digits)
*   Parameters :    Value between (0-999) to be encoded into BCD 
*   Purpose :       Convert an unsigned integer into a BCD encoded character.
*                   The input must be in the range 0 to 1023.
*                   The result is an integer where the three lowest nibbles
*                   contain the ones, tens and hundreds of the input.
*
*                   ein Int als 3 Dezimalziffern senden (nur 0-1023 sinnvoll)
*                     0 (=0x00) als 0 0 0 senden
*                     1 (=0x01) als 0 0 1 senden
*                     9 (=0x09) als 0 0 9 senden
*                    10 (=0x0A) als 0 1 0 senden
*                    15 (=0x0F) als 0 1 5 senden
*                    16 (=0x10) als 0 1 6 senden
*                    99 (=0x63) als 0 9 9 senden
*                   100         als 1 0 0 senden
*                   255 (=0xFF) als 2 5 5 senden
*                   256 (=0x100)als 2 5 6 senden char geht nur bis 255, int bis 65536
*                   999         als 9 9 9 senden  
*                  1000         als A 0 0 senden damit es in 3 Stellen passt
*                  1001         als A 0 1 senden
*                  1009         als A 0 9 senden
*                  1023 (=0x3FF)als A 2 3 senden
*****************************************************************************/
unsigned int INT2BCD3(unsigned int input)
{
	// unsigned int 0-999 umwandeln in 000-999
    // UNIT_HUNDRED=2 z.B. 999 wandeln -> convertBCD(2,999) 
         
	return (unsigned int) convertBCD(UNIT_HUNDRED, input);
}


/*****************************************************************************
*   Function name : INT2BCD4
*   Returns :       Binary coded decimal value of the input (4 digits)
*   Parameters :    Value between (0-9999) to be encoded into BCD 
*   Purpose :       Convert an unsigned integer into a BCD encoded character.
*                   The input must be in the range 0 to 9999.
*
*                   ein Int als 4 Dezimalziffern senden (nur 0-9999 sinnvoll)
*                     0 (=0x00) als 0 0 0 0 senden
*                     1 (=0x01) als 0 0 0 1 senden
*                     9 (=0x09) als 0 0 0 9 senden
*                    10 (=0x0A) als 0 0 1 0 senden
*                    15 (=0x0F) als 0 0 1 5 senden
*                    16 (=0x10) als 0 0 1 6 senden
*                    99 (=0x63) als 0 0 9 9 senden
*                   100         als 0 1 0 0 senden
*                   255 (=0xFF) als 0 2 5 5 senden
*                   256 (=0x100)als 0 2 5 6 senden int bis 65536
*                   999         als 0 9 9 9 senden
*                  9999         als 9 9 9 9 senden
*                 10000         als A 0 0 0 senden damit es in 4 Stellen passt
*                 10001         als A 0 0 1 senden
*                 10009         als A 0 0 9 senden
*****************************************************************************/
unsigned int INT2BCD4(unsigned int input)
{
    // unsigned int 0-9999 umwandeln in 0000-9999 (passt noch in 16bit)
    // UNIT_THOU=1 z.B. 9999 wandeln -> convertBCD(1,9999) 
         
	return (unsigned int) convertBCD(UNIT_THOU, input);
}


/*****************************************************************************
*   Function name : INT2BCD5
*   Returns :       Binary coded decimal value of the input (4 digits)
*   Parameters :    Value between (0-65535) to be encoded into BCD 
*   Purpose :       Convert an unsigned integer into a BCD encoded character.
*                   The input must be in the range 0 to 65535.
*
*                   ein Int als 5 Dezimalziffern senden (nur 0-65535 sinnvoll)
*                     0 (=0x00) als 0 0 0 0 0 senden
*                     1 (=0x01) als 0 0 0 0 1 senden
*                     9 (=0x09) als 0 0 0 0 9 senden
*                    10 (=0x0A) als 0 0 0 1 0 senden
*                    15 (=0x0F) als 0 0 0 1 5 senden
*                    16 (=0x10) als 0 0 0 1 6 senden
*                    99 (=0x63) als 0 0 0 9 9 senden
*                   100         als 0 0 1 0 0 senden
*                   255 (=0xFF) als 0 0 2 5 5 senden
*                   256 (=0x100)als 0 0 2 5 6 senden int bis 65536
*                   999         als 0 0 9 9 9 senden
*                  9999         als 0 9 9 9 9 senden
*                 65535         als 6 5 5 3 5 senden
*****************************************************************************/
unsigned long INT2BCD5(unsigned int input)
{
	// unsigned int 0-65535 umwandeln in 00000-65535 (passt nicht mehr in 16bit)
	// UNIT_TEN_THOU=0 z.B. 33333 wandeln -> convertBCD(0,33333)  
	
	return (unsigned long) convertBCD(UNIT_TEN_THOU,input);
}


/*
unsigned long LONG2BCD5(unsigned long input)
{
	// unsigned long 0-99999 umwandeln in 0-99999
	// UNIT_TEN_THOU=0 z.B. 99999 wandeln -> convertBCD(0,99999)  
	
	return (unsigned long) convertBCD(UNIT_TEN_THOU,input);
}
*/
