//***************************************************************************
//  File........: ds1302.c
//  Target(s)...: ATmega328p
//  Compiler....: AVR-GCC
//  Description.: RTC routines for DS1302 real time clock
//***************************************************************************
/*  3.9.2016 Ueberarbeitung Treiber, Test ok.
             Init RTC klappt
             Auslesen RTC klappt
             Ausgabe ueber USART klappt
             String-Ausgabe ueber USART klappt
             Burst-Mode eingebaut -> 7 Bytes rasch hintereinander lesen
             Read-Burst Test ok.
*/

#include <stdint.h>  // Integer-Datentypen 8-64bit
#include <avr/io.h>  // Pins und Interrupt-Tabelle -> siehe auch iom328p.h
#include "main.h"    // CPU-Frequenz, sbi, cbi
//#include "bcd.h"   // BCD-Umwandlungen fuer Ausgabe
#include "delay.h"   // Wartezeiten 2us, ms
#include "ds1302.h"  // RTC-Pins, RTC-Funktionen + Prototypen
#include "usart.h"   // USART-Funktionen
#include <avr/pgmspace.h> // PSTR
//#include <stdio.h>
//#include <stdlib.h>

// Funktionsuebergeordnet

/*
void RTC_set_time_date(void) // set RTC to time, date. Dies arbeitet korrekt !
{

    // RTC setzen: Datum, Uhrzeit. Einmal nur noetig nach Batteriewechsel
	RTC_write_byte(w_protect,0x00);	// RTC-Schreibschutz aufheben, falls gesetzt
	RTC_write_byte(year_w,0x16);    // Jahr 2016=16
	RTC_write_byte(day_w,sat);      // Tag Sonntag=sun
	RTC_write_byte(month_w,0x09);   // Monat Sept=9
	RTC_write_byte(date_w,0x10);    // Datum 04.=04
	RTC_write_byte(hour_w,0x03);    // Stunde 21=21
	RTC_write_byte(min_w,0x44);     // Minuten 20=20
	RTC_write_byte(sec_w,0x00);     // Sekunden 00=00

} */

// set RTC to time, date
void RTC_set_time_date(unsigned char date,  // 0x31 31.12.15 20:17:00
                       unsigned char month, // 0x12
                       unsigned char year,  // 0x15
                       unsigned char hour,  // 0x20
                       unsigned char min,   // 0x17
                       unsigned char sec)   // 0x00
{

    // RTC setzen: Datum, Uhrzeit. Einmal nur noetig nach Batteriewechsel
	RTC_write_byte(w_protect,0x00);	// RTC-Schreibschutz aufheben, falls gesetzt
	RTC_write_byte(year_w,year);    // Jahr 2016=16
	//RTC_write_byte(day_w,sat);      // Tag Sonntag=sun - muesste ohne gehen?
	RTC_write_byte(month_w,month);   // Monat Sept=9
	RTC_write_byte(date_w,date);    // Datum 04.=04
	RTC_write_byte(hour_w,hour);    // Stunde 21=21
	RTC_write_byte(min_w,min);     // Minuten 20=20
	RTC_write_byte(sec_w,sec);     // Sekunden 00=00

}

/*
void RTC_Tx_send(void)  // send RTC to USART
{
    USART_Tx_msg(PSTR("\nRTC: ")); // \nRTC: ausgeben
    //USART_Tx('\n');          // \n ausgeben
    //USART_Tx('R');           // R ausgeben
    //USART_Tx('T');           // T ausgeben
    //USART_Tx('C');           // C ausgeben
    //USART_Tx(':');           // : ausgeben
    //USART_Tx(' ');           // Space ausgeben
    temp= RTC_read_byte(date_r);  // Datum 03.=03
    USART_Tx_byte(temp);          // Datum ausgeben
    USART_Tx('.');           // . ausgeben
    temp= RTC_read_byte(month_r); // Monat Sept=9
    USART_Tx_byte(temp);          // Monat ausgeben
    USART_Tx('.');           // . ausgeben
    temp= RTC_read_byte(year_r);  // Jahr 2016=16
    USART_Tx_byte(temp);          // Jahr ausgeben
    USART_Tx(' ');           // Space ausgeben
    temp= RTC_read_byte(hour_r);  // Stunde 11=11
    USART_Tx_byte(temp);          // Stunde ausgeben
    USART_Tx(':');           // : ausgeben
    temp= RTC_read_byte(min_r);   // Minuten 40=40
    USART_Tx_byte(temp);          // Minuten ausgeben
    USART_Tx(':');           // : ausgeben
    temp= RTC_read_byte(sec_r);	 // Sekunden 00=00
    USART_Tx_byte(temp);          // Sekunden ausgeben
    USART_Tx('\n');          // neue Zeile ausgeben

*/
/*
unsigned char RTC_get_hours(void) // read hours from RTC
{
	unsigned char i;
	unsigned char r_byte;
	unsigned char temp;

	unsigned char b10; // ggf. globale Hilfsvariable (Stunden 10add)
    unsigned char bpm; // ggf. globale Hilfsvariable (Stunden 20add/AM/PM)

	RTC_reset();   // definierter Zustand der Ansteuerpins

	// Kommando senden zum Auslesen Stunden
	RTC_write(hour_r); // erst Kommando senden, dann erst Daten einlesen

	//sbi(DDRB,io_ds1302);  // IO-Pin auf Ausgabe schalten: wozu? der DS sendet doch auch -> Konflikt
    //cbi(PORTB,io_ds1302); // IO-Pin auf Low schalten: wozu? nun treibt die CPU low gegen den DS-Baustein

	cbi(DDRB,io_ds1302);  // IO-Pin auf Eingabe schalten

	r_byte = 0x00;        // Empfangsbyte ruecksetzen

	// untere 4 bit D0-D3 der Stunden einlesen
	for(i = 0; i < 4; i++) //get the first 4 bits
                           // 0-9 1er Stunden (00-23)
	{
        temp = 0;
        if(bit_is_set(PINB,io_ds1302))
           temp = 1;
		temp <<= 7;
		r_byte >>= 1;
		r_byte |= temp;  // Stunden in R_Byte aufgebaut

		// positiver CLK-Puls - 2us high, 2us low
		sbi(PORTB,clk_ds1302);  // CLK auf high
		delay_us(2);            // 2us lassen (0.5MHz)
		cbi(PORTB,clk_ds1302);  // CLK auf low
		delay_us(2);            // 2us lassen
	}

    // bit D4 der Stunden einlesen
    b10 = 0;   // 10 Stunden addieren oder nicht
	if(bit_is_set(PINB,io_ds1302))
        b10 = 1;  // ja - 10h addieren

	// positiver CLK-Puls - 2us high, 2us low
    sbi(PORTB,clk_ds1302);  // CLK auf high
	delay_us(2);            // 2us lassen (0.5MHz)
	cbi(PORTB,clk_ds1302);  // CLK auf low
	delay_us(2);            // 2us lassen

    // bit D5 der Stunden einlesen
    bpm = 0;   // 20 Stunden addieren oder nicht bzw. PM-Anzeige
	if(bit_is_set(PINB,io_ds1302))
        bpm = 1;  // ja - 20h addieren (bei 24h-Zeit), sonst PM-Anzeige

    // wozu Auswertung b10 und bpm wenn dies dann nicht benutzt wird?

    // wozu noch diese Impulse wenn nichts ausgewertet wird?
    // braucht der chip diese Pulse?

	// positiver CLK-Puls - 2us high, 2us low
    //sbi(PORTB,clk_ds1302);  // CLK auf high
	//delay_us(2);            // 2us lassen (0.5MHz)
	//cbi(PORTB,clk_ds1302);  // CLK auf low
	//delay_us(2);            // 2us lassen

	cbi(PORTB,rst_ds1302);  // CE auf low
	//cbi(PORTB,clk_ds1302);  // SCLK auf low: wozu?

	r_byte	>>= 4;

	return r_byte;
}
*/

// Function for FatFs module
unsigned long RTC_get_fattime(void) // RTC im Burst lesen und packen fuer SD-Karte
{
	unsigned char sec,min,hour,date,month,day,year;

    RTC_reset();  // definierter Zustand der RTC-Ansteuerpins

    RTC_write(c_burst_r); // Register schreiben fuer RTC-Auslesen mittels Burst
    USART_Tx_msg(PSTR("\nRTC-Fattime: ")); // \nRTC: ausgeben
    sec = RTC_read();     // Auslesen Sekunden 0-59 in Variable
    min = RTC_read();     // Auslesen Minuten 0-59 in Variable
    hour = RTC_read();    // Auslesen Stunden 0-23 in Variable
    date = RTC_read();    // Auslesen Tage 1-31 in Variable
    month = RTC_read();   // Auslesen Monat 1-12 in Variable
    day = RTC_read();     // Auslesen Wochentag 1-7 in Variable
    year = RTC_read();    // Auslesen Jahr 0-99 in Variable

	//if (!RtcOk) return 0; // falls RTC nicht ok

	// pack date and time into 4 byte DWORD variable
	// xxxx xxxx xxxx xxxx xxxx xxxx xxxx xxxx  Variable 32bit
	// xxxx xxx                                 Jahr 0-127 ab 1980
	//         x xxx                            Monat 1-12
	//              x xxxx                      Tag 1-31
	//                     xxxx x               Stunde 0-24
	//                           xxx xxx        Minute 0-59
	//                                  x xxxx x  Sekunde 0-59 Sekunde fehlt !
	return	  ((unsigned long)(2000 + year - 1980) << 25)
			| ((unsigned long)month << 21)
			| ((unsigned long)date << 16)
			| ((unsigned long)hour << 11)
			| ((unsigned long)min << 5)
			| ((unsigned long)sec >> 1);
}


void RTC_read_burst(void) // alle 7 bytes der RTC rasch lesen
{
    unsigned char sec,min,hour,date,month,day,year;

    RTC_reset();  // definierter Zustand der Ansteuerpins

    RTC_write(c_burst_r); // Register schreiben fuer Auslesen mittels Burst
    USART_Tx_msg(PSTR("\nRTC-Burst: ")); // \nRTC: ausgeben
    sec = RTC_read();     // Auslesen Sekunden 0-59 in Variable
    min = RTC_read();     // Auslesen Minuten 0-59 in Variable
    hour = RTC_read();    // Auslesen Stunden 0-23 in Variable
    date = RTC_read();    // Auslesen Tage 1-31 in Variable
    month = RTC_read();   // Auslesen Monat 1-12 in Variable
    day = RTC_read();     // Auslesen Wochentag 1-7 in Variable
    year = RTC_read();    // Auslesen Jahr 0-99 in Variable

    USART_Tx_byte(date);  // Datum ausgeben
    USART_Tx('.');        // . ausgeben
    USART_Tx_byte(month); // Monat ausgeben
    USART_Tx('.');        // . ausgeben
    USART_Tx_byte(year);  // Jahr ausgeben
    USART_Tx(' ');        // Space ausgeben
    USART_Tx_byte(hour);  // Stunde ausgeben
    USART_Tx(':');        // : ausgeben
    USART_Tx_byte(min);   // Minuten ausgeben
    USART_Tx(':');        // : ausgeben
    USART_Tx_byte(sec);   // Sekunden ausgeben
    USART_Tx('\n');       // neue Zeile ausgeben
}

unsigned char RTC_read_byte(unsigned char w_byte_reg) // read byte with register w_byte_reg
{
	unsigned char temp;

	RTC_reset();  // definierter Zustand der Ansteuerpins

	RTC_write(w_byte_reg); // Register schreiben fuer Auslesen
	temp = RTC_read();     // Auslesen in Variable

	cbi(PORTB,rst_ds1302);  // CE auf low
	cbi(PORTB,clk_ds1302);  // SCLK auf low

	return temp;
}

void RTC_write_byte(unsigned char w_byte_reg, unsigned char w_byte_data) //write w_byte_date with register w_byte_reg
{
	RTC_reset();  // definierter Zustand der Ansteuerpins

	RTC_write(w_byte_reg);
	RTC_write(w_byte_data);

	cbi(PORTB,rst_ds1302);  // CE auf low
	cbi(PORTB,clk_ds1302);  // SCLK auf low
}

void RTC_reset(void)		// sets pins to begin and end ds1302 communication
{
	sbi(DDRB,rst_ds1302);   // setzt CE-Pin auf Ausgang
	sbi(DDRB,clk_ds1302);   // setzt CLK-Pin auf Ausgang
	cbi(PORTB,clk_ds1302);  // setzt CLK-Pin low
	cbi(PORTB,rst_ds1302);  // setzt CE-Pin low
	sbi(PORTB,rst_ds1302);  // setzt CE-Pin high
}

void RTC_write(unsigned char w_byte) // writes w_byte to DS1302
{
	unsigned char i;
	//DDRB = 0xFF;      // Vorsicht ! setzt alle PortB-Pins auf Ausgabe. Nur 3 Pins sind die RTC !
	sbi(DDRB,io_ds1302); // IO-Pin auf Ausgabe

	for(i = 0; i < 8; i++)  // 8 bits ausgeben
	{
		cbi(PORTB,io_ds1302);  // IO-Pin auf low
		if(w_byte &0x01)       // wenn hinterstes Bit=1
           sbi(PORTB,io_ds1302); // IO-Pin auf high

		// positiver CLK-Puls - 2us high, 2us low
        sbi(PORTB,clk_ds1302);  // CLK auf high
	    delay_us(2);            // 2us lassen (0.5MHz)
	    cbi(PORTB,clk_ds1302);  // CLK auf low
	    delay_us(2);            // 2us lassen

		w_byte >>=1;
	}
}

unsigned char RTC_read(void)		//reads ds1302 reply
{
	unsigned char i;
	unsigned char r_byte, r_byte2, temp;

	//sbi(DDRB,io_ds1302);  // IO-Pin auf Ausgabe: wozu?

	r_byte = 0x00;
	r_byte2 = 0x00;

	//cbi(PORTB,io_ds1302); // IO-Pin low setzen: wozu?
    cbi(DDRB,io_ds1302);  // IO-Pin auf Input

	// untere 4 bit D0-D3 einlesen
	for(i = 0; i < 4; i++) //get the first 4 bits
                           // 0-9 1er Sekunden (00-59) bzw.
                           // 0-9 1er Minuten (00-59) bzw.
                           // 0-9 1er Stunden (00-23) bzw.
                           // 1-9 1er Datum (01-31) bzw.
                           // 1-9 1er Monat (01-12) bzw.
                           // 1-7 1er Wochentag (01-07) bzw.
                           // 0-9 1er Jahr (00-99)
	{
        temp = 0;
		if(bit_is_set(PINB,io_ds1302))
            temp = 1;
		temp <<= 7;
		r_byte >>= 1;
		r_byte |= temp;

		// positiver CLK-Puls - 2us high, 2us low
		sbi(PORTB,clk_ds1302);  // CLK auf high
		delay_us(2);            // 2us lassen (0.5MHz)
		cbi(PORTB,clk_ds1302);  // CLK auf low
		delay_us(2);            // 2us lassen
	}

	// obere 4 bit D4-D7 einlesen
	for(i = 0; i < 4; i++) //get the next 4 bits
                           // 0-5 10er Sekunden (00-59) bzw.
                           // 0-5 10er Minuten (00-59) bzw.
                           // 0-2 10er Stunden (00-23) bzw.
                           // 0-3 10er Datum (01-31) bzw.
                           // 0-1 10er Monat (01-12) bzw.
                           // 0-0 10er Wochentag (01-07) bzw.
                           // 0-9 10er Jahr (00-99)
	{
        temp = 0;
		if(bit_is_set(PINB,io_ds1302))
            temp = 1;
		temp <<= 7;
		r_byte2 >>= 1;
		r_byte2 |= temp;

		// positiver CLK-Puls - 2us high, 2us low
		sbi(PORTB,clk_ds1302);  // CLK auf high
		delay_us(2);            // 2us lassen (0.5MHz)
		cbi(PORTB,clk_ds1302);  // CLK auf low
		delay_us(2);            // 2us lassen
	}
	r_byte >>= 4;
	r_byte2 >>= 4;
	r_byte = (r_byte2 * 10) + r_byte;

	return r_byte;
}
