// SD-CardReader-Projekt   Dr. Weisser 4.9.2016  7.9.2016
//***************************************************************************
//  File........: main.c
//  Target(s)...: ATmega328p
//  Compiler....: AVR-GCC
//  Description.: SD-CardReader ueber ATmega328p auf Arduino Nano3
//                liest ggf. Uhr aus mit "L" ueber UART
//                startet ggf. ADC mit "S" ueber UART
//
//                sendet die Werte ueber UART zurueck mit 115kbd
//                bersetzt mit Makefile: make clean, make all
//                per USB auf Arduino-nano3 geladen: make program
//***************************************************************************
/*  4.9.2016 RTC 2 Pins umgelegt auf PB0 (D8), PB1 (D9) in ds1302.h und main.c
             laeuft so. Uhr neu gestellt. Ok.
    6.9.2016 FatFS: FAT File System von ChaN begonnen
             RTC init, UART init,
             Make all, Make program
             main.c, main.h, usart.c, usart.h, bcd.c, bcd.h,
             ds1302.c, ds1302.h, ff.c, ff.h, sdmm.c
			 int.c fuer Abfangen versehentlicher Interrupts
			 Makefile fuer Erzeugen des Programms und Hineinladen
			 delay.h fuer genauere Zeiten, Test war bei 5s und 6s ok.
	7.9.2016 Anpassen SPI-Pins, uebersetzen klappt weitgehend
             get_fattime in ff.c ggf. anpassen
             erst mal ausschalten mit #define _FS_NORTC	1  in ffconf.h
             Codepage 437
    9.9.2016 Einbau #define DEBUG, geaendert auf delay_us
             Test klappt. COM10, 115kbd, HTERM-Ausgabe:
             f_mount: FRESULT 00
             disk_initialize2843: stat 00
             f_open: find_volume: RESULT 00
             f_open: FRESULT 00
             f_write: FRESULT 00
             f_close: FRESULT 00
             Daten erfolgreich geschrieben !
   10.9.2016 neue Funktion RTC_get_fattime() fuer SD-Karte
             Schreiben ohne RTC:
             Datei NEWFILE.TXT 31.12.2015 23:00 Inhalt: "It works!" ok !
             warum liegt dies 1 Stunde versetzt? Codepage?
             #define _FS_NORTC	0 -> RTC nun aktiv ausgewertet:
             Datei NEWFILE.TXT 10.9.2015 3:46 Inhalt: "It works!" ok !
             die RTC klappt nun.
             Setzen der RTC umgeschrieben. Klappt.
   11.9.2016 neue Funktion rcvr_mmc_slow() damit Karte bis zur
             Init langsam angesprochen werden kann und dann schnell
             disk_init () versagt. oft rcvr_mmc_slow() loest das Problem
             f_open: find_volume versagt dann. Result 01.
             leider klappt es scheinbar nicht teilweise
             langsam und teilweise schnell zu empfangen.
             schnell empfangen versagt - wie laesst sich das weiter beschleunigen?


*/

#include <stdint.h>   // Integer-Datentypen 8-64bit
#include <avr/io.h>   // Pins und Interrupt-Tabelle -> siehe auch iom328p.h
#include <avr/interrupt.h> // Interrupts
#include "main.h"     // CPU-Frequenz, sbi, cbi
#include "usart.h"    // USART-Funktionen
//#include "adc.h"    // ADC-Funktionen
#include "delay.h"    // Delay-Funktionen us, ms
#include "ds1302.h"   // RTC-Funktionen
//#include <stdio.h>  // fuer printf - wo geht das hin?
#include <avr/pgmspace.h> // PSTR
#include "ff.h"		  // SD-Card-Treiber
//#include <avr/sleep.h>
//#include <avr/wdt.h>
//#include "LCD_functions.h"
//#include "LCD_driver.h"
//#include "RTC.h"
//#include "timer0.h"
//#include "bcd.h"
//#include "dataflash.h"

#define DEBUG 1

/* nur fuer Test der SD-Karten-Pins */
#if DEBUG
#define DO_INIT()	DDRB  &= 0xEF   // 11101111 PB4: Initialize port for MMC DO as input
#define DO			PINB  &	 0x10	// 00010000 PB4: Test for MMC DO ('H':true, 'L':false)

#define DI_INIT()	DDRB  |= 0x08	// 00001000 PB3: Initialize port for MMC DI as output
#define DI_H()		PORTB |= 0x08	// 00001000 PB3: Set MMC DI "high"
#define DI_L()		PORTB &= 0xF7	// 11110111 PB3: Set MMC DI "low"

#define CK_INIT()	DDRB  |= 0x20	// 00100000 PB5: Initialize port for MMC SCLK as output
#define CK_H()		PORTB |= 0x20	// 00100000 PB5: Set MMC SCLK "high"
#define	CK_L()		PORTB &= 0xDF	// 11011111 PB5: Set MMC SCLK "low"

#define CS_INIT()	DDRB  |= 0x04	// 00000100 PB2: Initialize port for MMC CS as output
#define	CS_H()		PORTB |= 0x04	// 00000100 PB2: Set MMC CS "high"
#define CS_L()		PORTB &= 0xFB	// 11111011 PB2: Set MMC CS "low"
#endif // DEBUG

// Global variables
//

// Private functions
//
//void DF_write_int(int data);
//int DF_read_int(void);

// Init FAT-Filesystem fuer SD-Karte
FATFS FatFs;    // FatFs work area needed for each volume
FIL Fil;		// File object needed for each open file

/*****************************************************************************
*   Function name : main
*   Returns :       None
*   Parameters :    None
*   Purpose :       Contains the main loop of the program
*****************************************************************************/

int main(void)
{
    //uint8_t i; // char i;
    unsigned char sec, temp;  // RTC
	sec = 0;                  // RTC
	//unsigned char r;

    // set default values for the global variables

    // Initalisierung CPU, Pins in/out, ADC, USART
    Initialization();  // init CPU, Pins in/out, USART

#if DEBUG
    /*
    // Test CK-Card - ok - kommt am Kartenhalter an
    CK_INIT();
    while (1){
        CK_H();  // 125ns HI
        CK_L();  // 125ns + 125ns LO
    }

    // Test: erst Init Pins
    CS_INIT(); CS_H();		// Initialize port pin tied to CS
	CK_INIT(); CK_L();		// Initialize port pin tied to SCLK
	DI_INIT();				// Initialize port pin tied to DI
	DO_INIT();				// Initialize port pin tied to DO

    // Test:
    //BYTE r;  // byte einlesen von der Karte
    unsigned int bc=10000; //  10 byte einlesen = 80 dummyclocks
    DI_H();  // send 0xFF: dazu Schreibleitung HI
    USART_Tx_msg(PSTR("\nByte von Karte lesen: \n")); // Meldung ausgeben

    while(1){
    //do {
      r = 0;   if (DO) r++;  // bit7
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();        // 5x CK_H(), 4x CK_L() -> 1.2us ~800kHz
      CK_L();
      CK_L();
      r <<= 1; if (DO) r++;  // bit6
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();
      CK_L();
      CK_L();
      r <<= 1; if (DO) r++;  // bit5
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();
      CK_L();
      CK_L();
      r <<= 1; if (DO) r++;  // bit4
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();
      CK_L();
      CK_L();
      r <<= 1; if (DO) r++;  // bit3
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();
      CK_L();
      CK_L();
      r <<= 1; if (DO) r++;  // bit2
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();
      CK_L();
      CK_L();
      r <<= 1; if (DO) r++;  // bit1
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();
      CK_L();
      CK_L();
      r <<= 1; if (DO) r++;  // bit0
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_H();
      CK_L();        // Clk hi dann lo: je 120ns ~3.3MHz falls ohne Delay
      CK_L();
      CK_L();
      CK_L();

      //USART_Tx_msg(PSTR("hex: ")); // hex-Ausgabe
      //USART_Tx(r); // Byte von Karte ausgeben
      //USART_Tx_msg(PSTR("dezimal: ")); // Dezimal-Ausgabe
      //USART_Tx_byte(r); // Byte von Karte ausgeben
      //USART_Tx_msg(PSTR("\n")); // neue Zeile
    }
    //} while (--bc);

    // Test CS-Card - ok - kommt am Kartenhalter an
    CS_INIT();
    while (1){
        CS_H();
        CS_L();
    }

    // Test DI-Card - ok - kommt am Kartenhalter an
    DI_INIT();
    while (1){
        DI_H();
        DI_L();
    }

    // Test DI-Card - ok - kommt am Kartenhalter an
    DI_INIT();
    while (1){
        DI_H();
        DI_L();
    }

    // Test CK-Card - ok - kommt am Kartenhalter an
    CK_INIT();
    while (1){
        CK_H();
        CK_L();
    } */

    // Zeitstempel testen
    //unsigned long timestamp;
    //timestamp=GET_FATTIME();  // 4 byte holen
            // wenn keine RTC vorhanden oder aktiviert ist
            // mittels #define _FS_NORTC 1 dann wird ein fester
            // Wert verwendet aufgrund der Rechenvorschrift
            //  ((DWORD)(_NORTC_YEAR - 1980) << 25 |
            //   (DWORD)_NORTC_MON << 21 |
            //   (DWORD)_NORTC_MDAY << 16)
            // wobei nur Tag, Monat, Jahr verwendet werden.
            // so entsteht 1.1.2016 00:00:00
            // der PC macht daraus 31.12.2015 23:00.

#endif // DEBUG


    // Init FAT-Filesystem
    //FATFS fs;        // Work area (file system object) for the volume
    //BYTE buff[16];   // File read buffer
    //UINT br;         // File read count
    UINT bw;       // file write count

    FRESULT res;   // Petit FatFs function common result code
    /* FRESULT moegliche Rueckgabewerte:
    0 FR_OK 			  succeeded
	1 FR_DISK_ERR	      hard error occurred in the low level disk I/O layer
	2 FR_INT_ERR	      assertion failed
	3 FR_NOT_READY		  physical drive cannot work
	4 FR_NO_FILE          could not find file
	5 FR_NO_PATH		  could not find path
	6 FR_INVALID_NAME     path name format invalid
	7 FR_DENIED		      access denied due to prohibited access or directory full
	8 FR_EXIST			  access denied due to prohibited access
	9 FR_INVALID_OBJECT	  file/directory object is invalid
	10 FR_WRITE_PROTECTED physical drive write protected
	11 FR_INVALID_DRIVE	  logical drive number is invalid
	12 FR_NOT_ENABLED	  volume has no work area
	13 FR_NO_FILESYSTEM	  there is no valid FAT volume
	14 FR_MKFS_ABORTED	  f_mkfs() aborted due to any problem
	15 FR_TIMEOUT		  could not get grant to access the volume within defined period
	16 FR_LOCKED		  operation rejected according to the file sharing policy
	17 FR_NOT_ENOUGH_CORE LFN working buffer could not be allocated
	18 FR_TOO_MANY_OPEN_FILES number of open files > _FS_LOCK
	19 FR_INVALID_PARAMETER	  given parameter is invalid */

    // mount volume
    res=f_mount(&FatFs, "", 0);	// give work area to default drive
    //res=f_mount(&FatFs, "", 1);	// Test mit 1: give work area to default drive

#if DEBUG
    USART_Tx_msg(PSTR("\nf_mount: FRESULT ")); // Meldung ausgeben
    USART_Tx_byte(res);
    USART_Tx_msg(PSTR("\n")); // Meldung ausgeben
#endif // DEBUG

    // open file on card
    res=f_open(&Fil, "newfile.txt", FA_WRITE | FA_CREATE_ALWAYS); // create file

#if DEBUG
    USART_Tx_msg(PSTR("\nf_open: FRESULT ")); // Meldung ausgeben
    USART_Tx_byte(res);
    USART_Tx_msg(PSTR("\n")); // Meldung ausgeben
#endif // DEBUG

	if (res == FR_OK) {	// file geoeffnet und ok?
		res=f_write(&Fil, "It works!\r\n", 11, &bw);	// write data to file

#if DEBUG
		USART_Tx_msg(PSTR("\nf_write: FRESULT ")); // Meldung ausgeben
        USART_Tx_byte(res);
        USART_Tx_msg(PSTR("\n")); // Meldung ausgeben
#endif // DEBUG

		res=f_close(&Fil);								// close file

#if DEBUG
		USART_Tx_msg(PSTR("\nf_close: FRESULT ")); // Meldung ausgeben
        USART_Tx_byte(res);
        USART_Tx_msg(PSTR("\n")); // Meldung ausgeben
#endif // DEBUG

		if (bw == 11) {		// message if data written well
			USART_Tx_msg(PSTR("\nDaten erfolgreich geschrieben !\n")); // Meldung ausgeben
		}
	}

    sei();   // enable Interrupts
			 // USART-Interrupt aktiv (warte auf Start-Zeichen)
			 // undefinierte Interrupts abgefangen mit int.c


	// RTC ggf. setzen: Datum, Uhrzeit. ntig nach Batteriewechsel (siehe ds1302.c)
    //RTC_set_time_date(0x10,0x09,0x16,0x04,0x20,0x00); // 10.9.16 4:20:00

	while (1)
	{
		temp = sec;
		sec = RTC_read_byte(sec_r);	// read seconds
		if(sec != temp){
			//printf("seconds: %u", sec);
			USART_Tx_byte(sec);
			//printf("\r");
		}
		if(sec==10){

           // lese alle Register einzeln nacheinander aus
           USART_Tx_msg(PSTR("\nRTC: ")); // \nRTC: ausgeben
           temp= RTC_read_byte(date_r);   // Datum 03.=03
           USART_Tx_byte(temp);           // Datum ausgeben
           USART_Tx('.');           // . ausgeben
           temp= RTC_read_byte(month_r);  // Monat Sept=9
           USART_Tx_byte(temp);           // Monat ausgeben
           USART_Tx('.');           // . ausgeben
           temp= RTC_read_byte(year_r);   // Jahr 2016=16
           USART_Tx_byte(temp);           // Jahr ausgeben
           USART_Tx(' ');           // Space ausgeben
           temp= RTC_read_byte(hour_r);   // Stunde 11=11
           USART_Tx_byte(temp);           // Stunde ausgeben
           USART_Tx(':');           // : ausgeben
           temp= RTC_read_byte(min_r);    // Minuten 40=40
           USART_Tx_byte(temp);           // Minuten ausgeben
           USART_Tx(':');           // : ausgeben
           temp= RTC_read_byte(sec_r);	  // Sekunden 00=00
           USART_Tx_byte(temp);           // Sekunden ausgeben
           USART_Tx('\n');          // neue Zeile ausgeben

           // lese alle Register im Burst raschmoeglichst aus
           //RTC_read_burst();  // RTC im Burst lesen und seriell ausgeben


           delay_ms(1000);          // 1s Pause

		}

		//delay_ms(100);
		// blinke LED
        //PORTB |= (1<<PB5);   // schalte LED an PB5 ein
        //delay_ms(2500);      // ms Pause

        //USART_Tx_byte(sec);

        //PORTB &= ~(1<<PB5);  // schalte LED an PB5 aus
        delay_ms(10);      // ms Pause

	}

/*
    for (;;)   // Main loop
    {
        // blinke LED
        PORTB |= (1<<PB5);   // schalte LED an PB5 ein
        delay_ms(2500);      // ms Pause

        PORTB &= ~(1<<PB5);  // schalte LED an PB5 aus
        delay_ms(2500);      // ms Pause

    } //End Main loop
*/
    return 0;
}


/*****************************************************************************
*   Function name : Initialization
*   Returns :       None
*   Parameters :    None
*   Purpose :       Initialize different modules
*****************************************************************************/
void Initialization(void)
{
    //unsigned char tst;          // dummy

    //OSCCAL_calibration();       // calibrate the OSCCAL byte fr 7.3728 MHz bei Teiler 1

//    CLKPR = (1<<CLKPCE);        // set Clock Prescaler Change Enable
                                // Bitmuster von 1 wird um CLKPCE=7 Stellen nach links geschoben
                                // und dann ins Register CLKPR geschrieben. 1000000 also.

    // set prescaler = 1, 16 MHz lassen
//    CLKPR = (0<<CLKPS1) | (0<<CLKPS0);   // luft mit ca. 16 MHz
                          // Bitmuster 00000000 CLKPS1=1 Stelle nach links
                          // Bitmuster 00000000 CLKPS0=0 Stellen nach links
                          // Bitmuster verodern ergibt 00000000 nach CLKPR= Clk Div Faktor 1

    // Disable Analog Comparator (power save)
    ACSR = (1<<ACD);      // bit 7 = ACD im Analog Comp Status Register auf 1 setzen schaltet aus
                          // Bitmuster von 1 wird ACD=7 Stellen nach links geschoben. 1000000 also.
                          // spart Strom weil Analog Comp aus.

    // Disable Digital input on analog inputs (power save)
//    DIDR0 = (1<<ADC5D) | (1<<ADC4D) | (1<<ADC3D) | (1<<ADC2D) | (1<<ADC1D) | (1<<ADC0D); //
                          // Digital Input Disable Register 0
                          // schaltet digitale Eingangspuffer der als ADC benutzten Eingnge ab
                          // um Strom zu sparen -> kein digital Input dort

    // Disable Digital input on Ain0 und Ain1 (power save)
//    DIDR1 = (0<<AIN1D) | (0<<AIN0D); // Dig. Inputs werden gebraucht, also nicht Buffer abschalten
                          // Digital Input Disable Register 1

    // DDRB Pins teilweise auf Eingabe
    DDRB = (1<<DDB5) | (0<<DDB4) | (1<<DDB3) | (1<<DDB2) | (0<<DDB1) | (1<<DDB0);
                  // PB5 = SCK  = D13 rote LED auf board -> Ausgabe SCK an RTC   - out
                  // PB4 = MISO = D12 auf board -> ggf. Ein/Ausgang RTC einlesen - in
                  // PB3 = MOSI = D11 auf board -> ggf. Ausgabe RTC CE           - out
                  // PB2 = D10 auf board -> CS SD-Reader                         - out
                  // PB1 = D9  auf board -> ggf. Ein/Ausgang RTC einlesen        - in
                  // PB0 = D8  auf board -> ggf. Ausgabe RTC CE                  - out

    // PORTB pullup enable fuer Eingnge
    PORTB = (0<<PB5) | (1<<PB4) | (0<<PB3) | (0<<PB2) | (1<<PB1) | (0<<PB0);
	                            // PortB Daten Register: Enable pullup
                                // damit sind bei PortB-Input-Pins die pullups aktiv
                                // unbeschaltete Input-Pins nach high gezogen ->
                                // verursachen so keine erhhte Stromaufnahme,
                                // die auftreten wrde wenn Input-Pins floaten

    // DDRD Pins teilweise auf Eingabe
    DDRD = (0<<DDD7) | (0<<DDD6) | (0<<DDD5) | (0<<DDD4) | (0<<DDD3) | (0<<DDD2) | (1<<DDD1) | (0<<DDD0);
                  // PD7 =  - in
                  // PD6 =  - in
                  // PD5 =  - in
                  // PD4 =  - in
                  // PD3 =  - in
                  // PD2 =  - in
                  // PD1 = TX - out
                  // PD0 = RX - in

    // PORTD pullup enable fuer Eingnge
    PORTD = (1<<PD7) | (1<<PD6) | (1<<PD5) | (1<<PD4) | (1<<PD3) | (1<<PD2) | (0<<PD1) | (1<<PD0);
                                // PortD Daten Register: Enable pullup
                                // damit sind bei PortD-Input-Pins die pullups aktiv
                                // unbeschaltete Input-Pins nach high gezogen ->
                                // verursachen so keine erhhte Stromaufnahme,
                                // die auftreten wrde wenn Input-Pins floaten

//    RTC_init();                 // Start timer2 asynchronous, used for RTC clock
                                // Uhr luft mit 32.786 kHz

//    Timer0_Init();              // Used for

    // initialisiere UART lt. Table 19-12. Datenblatt ATmega328P S.203
	// bei 16MHz U2X0=1 38.4k -> 51 Fehler 0.2%
	//                  57.6k    34        0.8%
	//                  115.2k   16        2.1%
	//                  250k      7        0%
	//                  0.5M      3        0%
	//                  1M        1        0%
    //USART_init(51);  // serielle Schnittstelle mit 38.4kbd
	USART_init(16);  // serielle Schnittstelle mit 115.2kbd

	USART_Tx ('A');  // Zeichen zur Bestaetigung ruecksenden - klappt
	USART_Tx ('\n');  // neue Zeile - klappt auch

    // initialisiere AD-Wandler Ch0 Pin 23 fuer 5V-Bereich
//    ADC_init(5);  // 5V-Eingang

}
